/* Based of code from the author RWB.
   Modified by Malte Marwedel
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>

#define F_CPU 16000000L
#define BAUD_RATE   9600

void uart_init(void) {
  UBRR0H = (((F_CPU/BAUD_RATE)/16)-1)>>8;		// set baud rate
  UBRR0L = (((F_CPU/BAUD_RATE)/16)-1);
  /* Enable Tx and Rx */
  UCSR0B = (1<<RXEN0)|(1<<TXEN0);
  /* Set Frame: Data 8 Bit, No Parity and 1 Stop Bit */
  UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);
}

int uart_putch(char ch, FILE *stream) {
   if (ch == '\n')
    uart_putch('\r', stream);
   while (!(UCSR0A & (1<<UDRE)));
   UDR0=ch;
   return 0;
}

int uart_getch(FILE *stream) {
   unsigned char ch;
   while (!(UCSR0A & (1<<RXC0)));
   ch=UDR0;

   if (ch == '\r') {
     ch='\n';
   }
   /* Echo the output back to the terminal */
   uart_putch(ch,stream);
   return ch;
}

/* Assign I/O stream to UART */
FILE uart_str = FDEV_SETUP_STREAM(uart_putch, uart_getch, _FDEV_SETUP_RW);

int uart_main(void) {
  /* Define Output/Input Stream */
  stdout = stdin = &uart_str;
  /* Initiatl UART */
  uart_init();
  sei();                       // Enable Global Interrupt
	return 0;
}